@@ -14,6 +14,10 @@
 #include <stdlib.h> /* for getenv */
 #endif /* HAVE_STDLIB_H */
 
+#if HAVE_STDIO_H
+#include <stdio.h> /* for stderr */
+#endif /* HAVE_STDIO_H */
+
 /* term.h prototypes */
 extern int tgetent();
 extern int tgetflag();
@@ -250,15 +254,24 @@ static int add_keybindings( struct kui_map_set *map ) {
 static int import_keyseq(struct tlist *i, struct kui_map_set *map) {
     char *terminfo, *termcap;
     int ret;
+    static char *term_buffer = (char *)NULL;
+    static char *buffer = (char *)NULL;
+    char *env;
 
-    char *env = getenv("TERM");
+    if (term_buffer == 0)
+    {
+	   term_buffer = (char *)malloc(4080);
+	   buffer = (char *)malloc(4080);
+    }
+
+    env = getenv("TERM");
 
     if ( !env ) {
         fprintf ( stderr, "%s:%d TERM not set error", __FILE__, __LINE__);
         return -1;
     }
     
-    if ( ( ret = tgetent(NULL, env)) == 0 ) {
+    if ( ( ret = tgetent(term_buffer, env)) == 0 ) {
         fprintf ( stderr, "%s:%d tgetent 'No such entry' error", __FILE__, __LINE__);
         return -1;
     } else if ( ret == -1 ) {
@@ -267,11 +280,11 @@ static int import_keyseq(struct tlist *i, struct kui_map_set *map) {
     }
     
     /* Set up the termcap seq */ 
-    if ( (termcap = tgetstr(i->tname, NULL)) == 0 ) {
+    if ( (termcap = tgetstr(i->tname, &buffer)) == 0 ) {
         /*fprintf ( stderr, "CAPNAME (%s) is not present in this TERM's termcap description\n", i->tname);*/
-	} else if (termcap == (char*)-1 )
-        fprintf ( stderr, "CAPNAME (%s) is not a termcap string capability\n", i->tname);
-    else {
+	} else if (termcap == (char*)-1 ) {
+        /* fprintf ( stderr, "CAPNAME (%s) is not a termcap string capability\n", i->tname); */
+    } else {
 		if ( kui_ms_register_map ( map, termcap, i->cgdb_key_code ) == -1 ) 
 			return -1;
 	}
@@ -279,9 +292,9 @@ static int import_keyseq(struct tlist *i, struct kui_map_set *map) {
     /* Set up the terminfo seq */ 
     if ( (terminfo = tigetstr(i->tiname)) == 0 ) {
         /* fprintf ( stderr, "CAPNAME (%s) is not present in this TERM's terminfo description\n", i->tiname);*/
-	} else if (terminfo == (char*)-1 )
-        fprintf ( stderr, "CAPNAME (%s) is not a terminfo string capability\n", i->tiname);
-    else {
+	} else if (terminfo == (char*)-1 ) {
+       /* fprintf ( stderr, "CAPNAME (%s) is not a terminfo string capability\n", i->tiname); */
+    } else {
 		if ( kui_ms_register_map ( map, terminfo, i->cgdb_key_code ) == -1 ) 
 			return -1;
 	}
@@ -1250,7 +1263,7 @@ int kui_term_print_cgdb_key_array ( int *cgdb_key_array ) {
 	/* Display output buffer */
 	fprintf ( stderr, "CGDB_KEY_ARRAY(");
 
-	for ( i = 0; i < cgdb_key_array[i] != 0; i++ ) {
+	for ( i = 0; cgdb_key_array[i] != 0; i++ ) {
 		int is_ckey;
 
 		is_ckey = kui_term_is_cgdb_key ( cgdb_key_array[i] );
