@@ -107,13 +107,9 @@ static int command_do_finish( void );
 static int command_do_help( void );
 static int command_do_next( void );
 static int command_do_quit( void );
-static int command_do_quit_force( void );
 static int command_do_run( void );
 static int command_do_step( void );
-static int command_search_next( void );
 static int command_source_reload( void );
-static int command_start_tty( void );
-static int command_toggle_tty( void );
 
 static int command_parse_syntax( void );
 
@@ -134,16 +130,10 @@ static struct commands
     /* insert      */ { "insert",      command_focus_gdb },
     /* next        */ { "next",        command_do_next },
     /* q           */ { "q",           command_do_quit },
-    /* q!          */ { "q!",          command_do_quit_force },
     /* quit        */ { "quit",        command_do_quit },
-    /* quit!       */ { "quit!",       command_do_quit_force },
     /* run         */ { "run",         command_do_run },
-    /* search_next */ { "search_next", command_search_next },
-    /* start_tty   */ { "start_tty",   command_start_tty },
     /* step        */ { "step",        command_do_step },
     /* syntax      */ { "syntax",      command_parse_syntax },
-    /* toggle_tty  */ { "toggle_tty",  command_toggle_tty },
-    /* tty         */ { "tty",         command_focus_tty },
 };
 
 action_t get_command( const char *cmd )
@@ -281,16 +271,22 @@ int command_focus_tty( void )
 
 int command_do_continue( void )
 {
-    if ( tgdb_run_debugger_command ( tgdb, TGDB_CONTINUE ) == -1 ) 
+    tgdb_request_ptr request_ptr;
+    request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_CONTINUE);
+    if (!request_ptr)
         return -1;
+    handle_request (tgdb, request_ptr);
 
     return 0;
 }
 
 int command_do_finish( void )
 {
-    if ( tgdb_run_debugger_command ( tgdb, TGDB_FINISH ) == -1 )
+    tgdb_request_ptr request_ptr;
+    request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_FINISH);
+    if (!request_ptr)
         return -1;
+    handle_request (tgdb, request_ptr);
 
     return 0;
 }
@@ -303,8 +299,11 @@ int command_do_help( void )
 
 int command_do_next( void )
 {
-    if ( tgdb_run_debugger_command  ( tgdb, TGDB_NEXT ) == -1 )
+    tgdb_request_ptr request_ptr;
+    request_ptr = tgdb_request_run_debugger_command  (tgdb, TGDB_NEXT);
+    if (!request_ptr)
         return -1;
+    handle_request (tgdb, request_ptr);
 
     return 0;
 }
@@ -317,64 +316,46 @@ int command_do_quit( void )
     return 0;
 }
 
-int command_do_quit_force( void )
-{
-    cleanup();
-    exit(0);
-    return 0;
-}
-
 int command_do_run( void )
 {
     /* FIXME: see if there are any other arguments to pass to the run command */
-    if ( tgdb_run_debugger_command  ( tgdb, TGDB_RUN ) == -1 )
+    tgdb_request_ptr request_ptr;
+    request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_RUN);
+    if (!request_ptr)
         return -1;
+    handle_request (tgdb, request_ptr);
 
     return 0;
 }
 
 int command_do_step( void )
 {
-    if ( tgdb_run_debugger_command  ( tgdb, TGDB_STEP ) == -1 )
+    tgdb_request_ptr request_ptr;
+    request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_STEP);
+    if (!request_ptr)
         return -1;
+    handle_request (tgdb, request_ptr);
 
     return 0;
 }
 
-int command_search_next( void )
-{
-    if_search_next();
-    return 0;
-}
-
 int command_source_reload( void )
 {
     struct sviewer *sview = if_get_sview ();
 
+    if (!sview)
+      return -1;
+
+    /* If there is no current source file, then there is nothing to reload. */
+    if (!sview->cur)
+      return 0;
+
     if ( source_reload ( sview, sview->cur->path, 1 ) == -1 )
         return -1;
 
     return 0;
 }
 
-int command_start_tty( void )
-{
-    /* FIXME: Find out what this is supposed to do and implement */
-    return 1;
-//    if ( tgdb_new_tty() == -1 ) {
-//        return 1;
-//    } else {
-//        /* FIXME: interface does scr_free( tty_win ), tty_win = NULL, if_layout(); */
-//        return 0;
-//    }
-}
-
-int command_toggle_tty( void )
-{
-    if_tty_toggle();
-    return 0;
-}
-
 int command_parse_syntax( void )
 {
   /* This is something like: 
