@@ -659,6 +659,8 @@ static void decrease_win_height( int jump_or_tty ) {
 /* signal_handler: Handles the WINCH signal (xterm resize).
  * ---------------
  */
+
+void rl_sigint_recved (void);
 static void signal_handler(int signo) {
     extern int resize_pipe[2];
     if (signo == SIGWINCH){
@@ -668,8 +670,10 @@ static void signal_handler(int signo) {
             resize = 1;
         }
     } else if ( signo == SIGINT || signo == SIGTERM || 
-                signo == SIGQUIT || signo == SIGCHLD )
+                signo == SIGQUIT || signo == SIGCHLD ) {
+      rl_sigint_recved ();
 		tgdb_signal_notification ( tgdb, signo );
+    }
 }
 
 /* if_get_command: Gets a command from the user
@@ -904,6 +908,7 @@ toggle_breakpoint(struct sviewer *sview, enum tgdb_breakpoint_action t)
 {
   char *path;
   int   line;
+  tgdb_request_ptr request_ptr;
 
   if (!sview || !sview->cur || !sview->cur->path)
     return 0;
@@ -919,7 +924,13 @@ toggle_breakpoint(struct sviewer *sview, enum tgdb_breakpoint_action t)
   if ( sview->cur->buf.breakpts[line] )
     t = TGDB_BREAKPOINT_DELETE;
 
-  return tgdb_modify_breakpoint ( tgdb, path, line + 1, t );
+  request_ptr = tgdb_request_modify_breakpoint (tgdb, path, line + 1, t);
+  if (!request_ptr)
+    return -1;
+
+  handle_request (tgdb, request_ptr);
+  
+  return 0;
 }
 
 /* source_input: Handles user input to the source window.
@@ -986,7 +997,11 @@ static void source_input(struct sviewer *sview, int key)
            break;
         case 'o':
            /* Causes file dialog to be opened */
-           tgdb_get_inferiors_source_files (tgdb);
+	   {
+	     tgdb_request_ptr request_ptr;
+             request_ptr = tgdb_request_inferiors_source_files (tgdb);
+	     handle_request (tgdb, request_ptr);
+	   }
            break;
         case ':':
            /* Allows user to go to a line number */ 
@@ -1010,16 +1025,19 @@ static void source_input(struct sviewer *sview, int key)
 
     /* Some extended features that are set by :set sc */
     if ( shortcut_option ) {
+	tgdb_request_ptr request_ptr = NULL;
         switch ( key ) {
-            case 'r': tgdb_run_debugger_command (tgdb, TGDB_RUN); 		break;
-            case 'n': tgdb_run_debugger_command (tgdb, TGDB_NEXT);  	break;
-            case 's': tgdb_run_debugger_command (tgdb, TGDB_STEP);  	break;
-            case 'c': tgdb_run_debugger_command (tgdb, TGDB_CONTINUE);	break;
-            case 'f': tgdb_run_debugger_command (tgdb, TGDB_FINISH);  	break;
-            case 'u': tgdb_run_debugger_command (tgdb, TGDB_UP);      	break;
-            case 'd': tgdb_run_debugger_command (tgdb, TGDB_DOWN);    	break;
+            case 'r': request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_RUN);		break;
+            case 'n': request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_NEXT);	break;
+            case 's': request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_STEP);	break;
+            case 'c': request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_CONTINUE);	break;
+            case 'f': request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_FINISH);  	break;
+            case 'u': request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_UP);      	break;
+            case 'd': request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_DOWN);    	break;
             default:                                    				break;
         }
+	if (request_ptr)
+	  handle_request (tgdb, request_ptr);
     }
 
     if_draw();
@@ -1165,23 +1183,43 @@ int internal_if_input(int key) {
                      return 0;
                 case CGDB_KEY_F5:
                     /* Issue GDB run command */
-                    tgdb_run_debugger_command ( tgdb, TGDB_RUN);
+		    {
+		      tgdb_request_ptr request_ptr;
+                      request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_RUN);
+		      handle_request (tgdb, request_ptr);
+		    }
                     return 0;
                 case CGDB_KEY_F6:
                     /* Issue GDB continue command */
-                    tgdb_run_debugger_command (tgdb, TGDB_CONTINUE);
+		    {
+		      tgdb_request_ptr request_ptr;
+                      request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_CONTINUE);
+		      handle_request (tgdb, request_ptr);
+		    }
                     return 0;
                 case CGDB_KEY_F7:
                     /* Issue GDB finish command */
-                    tgdb_run_debugger_command (tgdb, TGDB_FINISH);
+		    {
+		      tgdb_request_ptr request_ptr;
+                      request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_FINISH);
+		      handle_request (tgdb, request_ptr);
+		    }
                     return 0;
                 case CGDB_KEY_F8:
                     /* Issue GDB next command */
-                    tgdb_run_debugger_command (tgdb, TGDB_NEXT);
+		    {
+		      tgdb_request_ptr request_ptr;
+                      request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_NEXT);
+		      handle_request (tgdb, request_ptr);
+		    }
                     return 0;
                 case CGDB_KEY_F10:
                     /* Issue GDB step command */
-                    tgdb_run_debugger_command (tgdb, TGDB_STEP);
+		    {
+		      tgdb_request_ptr request_ptr;
+                      request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_STEP);
+		      handle_request (tgdb, request_ptr);
+		    }
                     return 0;
                 case CGDB_KEY_CTRL_L:
                     if_layout();
@@ -1196,7 +1234,6 @@ int internal_if_input(int key) {
             return gdb_input(key);
         case FILE_DLG: {
             static char filedlg_file[MAX_LINE];
-            extern char last_relative_file[MAX_LINE];
             int ret = filedlg_recv_char(fd, key, filedlg_file);
             /* The user cancelled */
             if ( ret == -1 ) { 
@@ -1207,8 +1244,9 @@ int internal_if_input(int key) {
                 return 0;
             /* The user picked a file */
             } else if ( ret == 1 ) {
-                tgdb_get_absolute_path ( tgdb, filedlg_file);
-                strcpy(last_relative_file, filedlg_file);
+		tgdb_request_ptr request_ptr;
+                request_ptr = tgdb_request_filename_pair (tgdb, filedlg_file);
+		handle_request (tgdb, request_ptr);
                 if_set_focus(CGDB);
                 return 0;
             }
@@ -1363,24 +1401,6 @@ Focus if_get_focus(void) {
     return focus;
 }
 
-void if_tty_toggle( void )
-{
-    tty_win_on = !tty_win_on;
-    if ( tty_win_on ) {
-        if_set_focus( CGDB );
-    } else {
-        if_set_focus( TTY );
-    }
-
-    if_layout();
-}
-
-void if_search_next( void )
-{
-    source_search_regex(src_win, regex_line, 2, regex_direction, regex_icase);
-    if_draw();
-}
-
 void if_set_winsplit( WIN_SPLIT_TYPE new_split )
 {
    cur_win_split = new_split;
