@@ -62,7 +62,7 @@ enum annotate_commands {
 	/**
 	 * absolute source path.
 	 */
-    ANNOTATE_INFO_SOURCE_ABSOLUTE,
+    ANNOTATE_INFO_SOURCE_FILENAME_PAIR,
 
 	/**
  	 * Shows information on the current source file
@@ -74,6 +74,14 @@ enum annotate_commands {
 	 */
     ANNOTATE_LIST,
 
+	/**
+ 	 * Get's the current fullname, filename and line number.
+	 * This is because the 'info line' command to GDB generates the source
+	 * annotation, which in turn causes the a2 subsystem to get the 
+	 * relative path.
+	 */
+    ANNOTATE_INFO_LINE,
+
 	/**
 	 * Sets the prompt.
 	 */
@@ -322,10 +330,21 @@ struct tgdb_list *a2_get_client_commands ( void *ctx );
  * @return
  * 0 on success, otherwise -1 on error.
  */
-int a2_get_source_absolute_filename ( 
+int a2_get_source_filename_pair ( 
 		void *ctx, 
 		const char *file );
 
+/**
+ * Get's the fullname, filename and line number GDB is currently at.
+ *
+ * \param ctx
+ * The annotate two context.
+ *
+ * @return
+ * 0 on success, otherwise -1 on error.
+ */
+int a2_get_current_location (void *ctx);
+
 /** 
  * Gets all the source files that the inferior makes up.
  *
@@ -337,20 +356,6 @@ int a2_get_source_absolute_filename (
  */
 int a2_get_inferior_sources ( void *ctx );
 
-/** 
- * This will change the prompt the user sees to PROMPT.
- *
- * \param ctx
- * The annotate two context.
- *
- * \param prompt
- * The new prompt to change too.
- *
- * @return
- * 0 on success, otherwise -1 on error.
- */
-int a2_change_prompt( void *ctx, const char *prompt);
-
 /** 
  * This is called when readline determines a command needs to be completed.
  *
@@ -444,7 +449,7 @@ int a2_open_new_tty ( void *ctx, int *inferior_stdin, int *inferior_stdout );
  * @return
  * tty name on success, otherwise NULL on error.
  */
-char *a2_get_tty_name ( void *ctx );
+const char *a2_get_tty_name ( void *ctx );
 
 //@}
 
