@@ -11,22 +11,29 @@
 #endif /* HAVE_STDLIB_H */
 
 #include "annotate.h"
+#include "sys_util.h"
 #include "logger.h"
 #include "data.h"
 #include "commands.h"
 #include "globals.h"
 #include "io.h"
 
-static int handle_source(struct annotate_two *a2, const char *buf,  size_t n, struct tgdb_list *list){
-   int ret = commands_parse_source(
-		   a2->c, a2->client_command_list, buf, n, list);
+static int 
+handle_source (struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list)
+{
+  int ret = commands_parse_source (a2->c, a2->client_command_list, buf, n, list);
 
-   /* This tells the annotate subsystem if the source annotation has been
-	* reached. This is important because if the source annotation has been 
-	* reached before tgdb is initialized, then the annotate subsystem doesn't
-	* have to probe gdb for the first file to open.
-	*/
+  /* This tells the annotate subsystem if the source annotation has been
+   * reached. This could be useful to the application using TGDB to determine if a single
+   * source annotation has been reached yet. If so, it doesn't have to probe TGDB for the
+   * initiale source file when GDB starts up. 
+   *
+   * As of 12/17/2005 it is no longer being used within TGDB. I moved the responsibilty of
+   * probing for the first file to the calling application. If the variable ends up being
+   * useless, it can be removed.
+   */
    a2->source_already_received = 1;
+
    return ret;
 }
 
@@ -79,6 +86,13 @@ static int handle_prompt(struct annotate_two *a2, const char *buf, size_t n, str
       return 0;
    } 
 
+   /* 'complete' is done, return the completions to the gui */
+   if(global_has_completion_started(a2->g) == TRUE){
+      global_reset_completion_started(a2->g);
+      commands_send_gui_completions(a2->c, list);
+      return 0;
+   } 
+
    return 0;
 }
 
@@ -88,7 +102,8 @@ static int handle_post_prompt(struct annotate_two *a2, const char *buf, size_t n
 }
 
 static int handle_breakpoints_invalid(struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list){
-   return 0;
+  /* Don't use anymore because GDB is buggy */
+  return 0;
 }
 
 static int handle_breakpoints_headers(struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list){
@@ -106,6 +121,10 @@ static int handle_breakpoints_table_end(struct annotate_two *a2, const char *buf
    return 0;
 }
 
+static int handle_unwanted_fields(struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list){
+  return 0;
+}
+
 static int handle_field(struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list){
    int i = -1;
    
@@ -131,7 +150,10 @@ static int handle_error_begin(struct annotate_two *a2, const char *buf, size_t n
     * annotate error ( usually meaning that gdb can not find the symbols
     * for the debugged program ) then send a denied response. */
    if ( commands_get_state(a2->c) == INFO_SOURCES ) {
-        tgdb_types_append_command(list, TGDB_SOURCES_DENIED, NULL );
+        struct tgdb_response *response = (struct tgdb_response *)
+     	  xmalloc (sizeof (struct tgdb_response));
+	response->header = TGDB_SOURCES_DENIED;
+        tgdb_types_append_command (list, response);
         return 0;
    }
 
@@ -158,43 +180,20 @@ static int handle_quit(struct annotate_two *a2, const char *buf, size_t n, struc
    return 0;
 }
 
-static int handle_display_begin(struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list){
-   return 0;
-}
-
-static int handle_display_number_end(struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list){
-   return 0;
-}
-
-static int handle_display_format(struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list){
-   return 0;
-}
-
-static int handle_display_expression(struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list){
-   return 0;
-}
-
-static int handle_display_expression_end(struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list){
-   return 0;
-}
-
-static int handle_display_value(struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list){
-   return 0;
-}
-
 static int handle_exited(struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list){
-	char *tmp = (char*)malloc ( sizeof ( char )*(n+1));
-	int *i = (int *)malloc ( sizeof ( int ) );
-	sprintf ( tmp, buf + 7 ); /* Skip the 'exited ' part */
-	*i = atoi ( tmp ) ;
-	free ( tmp );
-	tmp = NULL;
-    tgdb_types_append_command(list, TGDB_INFERIOR_EXITED, i );
-   return 0;
-}
-
-static int NOT_SUPPORTED(struct annotate_two *a2, const char *buf, size_t n, struct tgdb_list *list){
-   return 0;
+  char *tmp = (char*)malloc ( sizeof ( char )*(n+1));
+  int *i = (int *)malloc ( sizeof ( int ) );
+  struct tgdb_response *response;
+  sprintf ( tmp, buf + 7 ); /* Skip the 'exited ' part */
+  *i = atoi ( tmp ) ;
+  free ( tmp );
+  tmp = NULL;
+  response = (struct tgdb_response *)
+    xmalloc (sizeof (struct tgdb_response));
+  response->header = TGDB_INFERIOR_EXITED;
+  response->choice.inferior_exited.exit_status = i;
+  tgdb_types_append_command(list, response);
+  return 0;
 }
 
 /**
@@ -319,69 +318,9 @@ static struct annotation {
       handle_breakpoints_table
   },
   {
-      "value-history-begin",
-      19,
-      NOT_SUPPORTED
-  },
-  {
-      "value-history-value",
-      19,
-      NOT_SUPPORTED
-  },
-  {
-      "value-history-end",
-      17,
-      NOT_SUPPORTED
-  },
-  {
-      "value-begin",
-      11,
-      NOT_SUPPORTED
-  },
-  {
-      "value-end",
-      9,
-      NOT_SUPPORTED
-  },
-  {
-      "arg-begin",
-      9,
-      NOT_SUPPORTED
-  },
-  {
-      "arg-name-end",
-      12,
-      NOT_SUPPORTED
-  },
-  {
-      "arg-value",
-      9,
-      NOT_SUPPORTED
-  },
-  {
-      "arg-end",
-      7,
-      NOT_SUPPORTED
-  },
-  {
-      "field-begin",
-      11,
-      NOT_SUPPORTED
-  },
-  {
-      "field-name-end",
-      14,
-      NOT_SUPPORTED
-  },
-  {
-      "field-value",
-      11,
-      NOT_SUPPORTED
-  },
-  {
-      "field-end",
-      9,
-      NOT_SUPPORTED
+      "field-",
+      6,
+      handle_unwanted_fields
   },
   {
       "field",
@@ -409,39 +348,9 @@ static struct annotation {
       handle_quit
   },
   {
-      "display-begin",
-      13,
-      handle_display_begin
-  },
-  {
-     "display-number-end",
-     18,
-     handle_display_number_end
-  },
-  {
-     "display-format",
-      14,
-      handle_display_format
-  },
-  {
-     "display-expression",
-     18,
-     handle_display_expression
-  },
-  {
-     "display-expression-end",
-     22,
-     handle_display_expression_end
-  },
-  {
-     "display-value",
-     13,
-     handle_display_value
-  },
-  {
-	  "exited",
-	  6,
-	  handle_exited
+      "exited",
+      6,
+      handle_exited
   },
   {
       NULL,
