@@ -21,7 +21,6 @@
 #include "queue.h"
 #include "ibuf.h"
 #include "a2-tgdb.h"
-#include "rlctx.h"
 #include "queue.h"
 #include "tgdb_list.h"
 #include "annotate_two.h"
@@ -97,6 +96,16 @@ struct commands {
 	 */
 	struct ibuf *info_source_string;
 
+	/**
+	 * The discovered relative path, found from the info source output.
+	 */
+	struct ibuf *info_source_relative_path;
+
+	/**
+	 * The discovered absolute path, found from the info source output.
+	 */
+	struct ibuf *info_source_absolute_path;
+
 	/** 
 	 * Finished parsing the line being looked for.
 	 */
@@ -109,7 +118,7 @@ struct commands {
 
 	//@}
 	
-	/** 'info sources' information */
+	// info sources information {{{
 	//@{
 
 	/** 
@@ -128,6 +137,28 @@ struct commands {
 	struct tgdb_list *inferior_source_files;
 
 	//@}
+	// }}}
+	
+	// tab completion information {{{
+	//@{
+
+	/** 
+	 * ??? Finished parsing the data being looked for.
+	 */
+	int tab_completion_ready;
+
+	/**
+	 * A tab completion item
+	 */
+	struct ibuf *tab_completion_string;
+
+	/**
+	 * All of the tab completion items, parsed in put in a list, 1 at a time.
+	 */
+	struct tgdb_list *tab_completions;
+
+	//@}
+	// }}}
 
 	/** 
 	 * The absolute path prefix output by GDB when 'info source' is given
@@ -150,62 +181,117 @@ struct commands {
 	int source_relative_prefix_length;
 };
 
-struct commands *commands_initialize(void) {
-	struct commands *c = (struct commands *)
-		xmalloc ( sizeof ( struct commands ));
+struct commands *
+commands_initialize(void)
+{
+  struct commands *c = (struct commands *)
+    xmalloc ( sizeof ( struct commands ));
 
-    c->absolute_path       	= ibuf_init();
-    c->line_number         	= ibuf_init();
+  c->absolute_path       	= ibuf_init();
+  c->line_number         	= ibuf_init();
 
-	c->cur_command_state 	= VOID_COMMAND;
-	c->cur_field_num 		   = 0;
-   c->field_5_newline_hit  = 0;
+  c->cur_command_state		= VOID_COMMAND;
+  c->cur_field_num		= 0;
+  c->field_5_newline_hit	= 0;
 
-    c->breakpoint_list    	= tgdb_list_init();
-    c->breakpoint_string   	= ibuf_init();
-	c->breakpoint_table    	= 0;
-	c->breakpoint_enabled   = FALSE;
-	c->breakpoint_started   = FALSE;
+  c->breakpoint_list    	= tgdb_list_init();
+  c->breakpoint_string   	= ibuf_init();
+  c->breakpoint_table    	= 0;
+  c->breakpoint_enabled		= FALSE;
+  c->breakpoint_started		= FALSE;
 
-    c->info_source_string  	= ibuf_init();
-	c->info_source_ready   	= 0;
-	c->last_info_source_requested = ibuf_init();
+  c->info_source_string  	= ibuf_init();
+  c->info_source_relative_path	= ibuf_init();
+  c->info_source_absolute_path	= ibuf_init();
+  c->info_source_ready		= 0;
+  c->last_info_source_requested	= ibuf_init();
 
-	c->sources_ready 		= 0;
-    c->info_sources_string 	= ibuf_init();
-	c->inferior_source_files= tgdb_list_init ();
+  c->sources_ready 		= 0;
+  c->info_sources_string 	= ibuf_init ();
+  c->inferior_source_files	= tgdb_list_init ();
 
-	c->source_prefix 		= "Located in ";
-	c->source_prefix_length = 11;
+  c->tab_completion_ready	= 0;
+  c->tab_completion_string 	= ibuf_init();
+  c->tab_completions		= tgdb_list_init ();
 
-	c->source_relative_prefix 	= "Current source file is ";
-	c->source_relative_prefix_length 	= 23;
+  c->source_prefix 		= "Located in ";
+  c->source_prefix_length	= 11;
 
-	return c;
+  c->source_relative_prefix 	= "Current source file is ";
+  c->source_relative_prefix_length = 23;
+
+  return c;
 }
 
-void commands_shutdown ( struct commands *c ) {
-	if ( c == NULL )
-		return;
+int free_breakpoint (void *item) {
+  struct tgdb_breakpoint *bp = (struct tgdb_breakpoint *)item;
+  if (bp->file) {
+    free (bp->file);
+    bp->file = NULL;
+  }
 
-	ibuf_free ( c->absolute_path );
-	c->absolute_path = NULL;
+  if (bp->funcname) {
+    free (bp->funcname);
+    bp->funcname = NULL;
+  }
 
-	ibuf_free ( c->line_number );
-	c->line_number = NULL;
+  free (bp);
+  bp = NULL;
 
-	/* TODO: free breakpoint queue */
+  return 0;
+}
 
-	ibuf_free ( c->breakpoint_string );
-	c->breakpoint_string = NULL;
+int free_char_star (void *item) {
+  char *s = (char *)item;
 
-	ibuf_free ( c->info_sources_string );
-	c->info_sources_string = NULL;
+  free (s);
+  s = NULL;
+
+  return 0;
+}
+
+void 
+commands_shutdown (struct commands *c)
+{
+  if (c == NULL)
+    return;
+
+  ibuf_free (c->absolute_path);
+  c->absolute_path = NULL;
+
+  ibuf_free (c->line_number);
+  c->line_number = NULL;
+
+  tgdb_list_free (c->breakpoint_list, free_breakpoint);
+  tgdb_list_destroy (c->breakpoint_list);
+
+  ibuf_free (c->breakpoint_string);
+  c->breakpoint_string = NULL;
+
+  ibuf_free (c->info_source_string);
+  c->info_source_string = NULL;
+
+  ibuf_free (c->info_source_relative_path);
+  c->info_source_relative_path = NULL;
+
+  ibuf_free (c->info_source_absolute_path);
+  c->info_source_absolute_path = NULL;
+
+  ibuf_free (c->info_sources_string);
+  c->info_sources_string = NULL;
+
+  ibuf_free (c->tab_completion_string);
+  c->tab_completion_string = NULL;
+
+  tgdb_list_destroy (c->tab_completions);
+
+  tgdb_list_free (c->inferior_source_files, free_char_star);
+  tgdb_list_destroy (c->inferior_source_files);
 	
-	/* TODO: free source_files queue */
+  /* TODO: free source_files queue */
 
-	free ( c );
-	c = NULL;
+  free ( c );
+  c = NULL;
 }
 
 int commands_parse_field(struct commands *c, const char *buf, size_t n, int *field){
@@ -402,13 +488,16 @@ void commands_set_state(
         case BREAKPOINT_TABLE_END:  
             if(ibuf_length(c->breakpoint_string) > 0)
                 parse_breakpoint ( c );
+	    {
+		struct tgdb_response *response = (struct tgdb_response *)
+		  xmalloc (sizeof (struct tgdb_response));
+		response->header = TGDB_UPDATE_BREAKPOINTS;
+		response->choice.update_breakpoints.breakpoint_list = c->breakpoint_list;
 
-            /* At this point, annotate needs to send the breakpoints to the gui.
-             * All of the valid breakpoints are stored in breakpoint_queue. */
-            tgdb_types_append_command ( 
-						list, 
-						TGDB_UPDATE_BREAKPOINTS, 
-						c->breakpoint_list );
+		/* At this point, annotate needs to send the breakpoints to the gui.
+		 * All of the valid breakpoints are stored in breakpoint_queue. */
+	        tgdb_types_append_command (list, response);
+	    }
 
             ibuf_clear(c->breakpoint_string);
             c->breakpoint_enabled = FALSE;
@@ -424,7 +513,7 @@ void commands_set_state(
             c->breakpoint_table = 1; 
             c->breakpoint_started = TRUE;
             break;
-        case INFO_SOURCE_ABSOLUTE:
+        case INFO_SOURCE_FILENAME_PAIR:
             break;
         case INFO_SOURCE_RELATIVE:
             break;
@@ -448,16 +537,20 @@ enum COMMAND_STATE commands_get_state( struct commands *c){
    return c->cur_command_state;
 }
 
-static void commands_prepare_info_source(struct annotate_two *a2, struct commands *c, enum COMMAND_STATE state){
-   data_set_state(a2, INTERNAL_COMMAND );
-   ibuf_clear(c->info_source_string);
+static void 
+commands_prepare_info_source (struct annotate_two *a2, struct commands *c, enum COMMAND_STATE state)
+{
+  data_set_state (a2, INTERNAL_COMMAND);
+  ibuf_clear (c->info_source_string);
+  ibuf_clear (c->info_source_relative_path);
+  ibuf_clear (c->info_source_absolute_path);
    
-   if ( state == INFO_SOURCE_ABSOLUTE ) {
-        commands_set_state(c, INFO_SOURCE_ABSOLUTE, NULL);
-   } else if ( state == INFO_SOURCE_RELATIVE )
-        commands_set_state(c, INFO_SOURCE_RELATIVE, NULL);
+  if (state == INFO_SOURCE_FILENAME_PAIR)
+    commands_set_state (c, INFO_SOURCE_FILENAME_PAIR, NULL);
+   else if (state == INFO_SOURCE_RELATIVE)
+    commands_set_state (c, INFO_SOURCE_RELATIVE, NULL);
 
-   c->info_source_ready = 0;
+  c->info_source_ready = 0;
 }
 
 void commands_list_command_finished (
@@ -466,74 +559,88 @@ void commands_list_command_finished (
 	int success){
   /* The file does not exist and it can not be opened.
    * So we return that information to the gui.  */
-	struct tgdb_source_file *rejected = (struct tgdb_source_file *)
-	  xmalloc ( sizeof ( struct tgdb_source_file ) );
+  struct tgdb_source_file *rejected = (struct tgdb_source_file *)
+    xmalloc ( sizeof ( struct tgdb_source_file ) );
+  struct tgdb_response *response = (struct tgdb_response *)
+    xmalloc (sizeof (struct tgdb_response));
 
-	if ( c->last_info_source_requested == NULL ) 
-		rejected->absolute_path = NULL;
-	else
-		rejected->absolute_path = strdup ( ibuf_get ( c->last_info_source_requested ) );
+  if ( c->last_info_source_requested == NULL ) 
+    rejected->absolute_path = NULL;
+  else
+    rejected->absolute_path = strdup ( ibuf_get ( c->last_info_source_requested ) );
 
-    tgdb_types_append_command ( list, TGDB_ABSOLUTE_SOURCE_DENIED, rejected);
+  response->header = TGDB_ABSOLUTE_SOURCE_DENIED;
+  response->choice.absolute_source_denied.source_file = rejected;
+
+  tgdb_types_append_command (list, response);
 }
 
-void commands_send_source_absolute_source_file (
-	struct commands *c, 
-	struct tgdb_list *list){
-   /*err_msg("Whats up(%s:%d)\r\n", info_source_buf, info_source_buf_pos);*/
-    unsigned long length = ibuf_length(c->info_source_string);
-    static char *info_ptr; 
-    info_ptr = ibuf_get(c->info_source_string);
+/* This will send to the gui the absolute path to the file being requested. 
+ * Otherwise the gui will be notified that the file is not valid.
+ */
+static void 
+commands_send_source_absolute_source_file (struct commands *c, 
+					   struct tgdb_list *list)
+{
+  /*err_msg("Whats up(%s:%d)\r\n", info_source_buf, info_source_buf_pos);*/
+  unsigned long length = ibuf_length(c->info_source_absolute_path);
 
-   /* found */
-   if(length >= c->source_prefix_length && 
-      (strncmp(info_ptr, c->source_prefix, c->source_prefix_length) == 0)){
-      char *path = info_ptr + c->source_prefix_length;
+  /* found */
+  if (length > 0)
+  {
+    char *apath = NULL, *rpath = NULL;
+    struct tgdb_response *response;
+    if (length > 0)
+      apath = ibuf_get (c->info_source_absolute_path);
+    if (ibuf_length (c->info_source_relative_path) > 0)
+      rpath = ibuf_get (c->info_source_relative_path);
 
-      /* requesting file */
-      if(c->last_info_source_requested == NULL) {
-		  	/* This happens only when libtgdb starts */
-            ibuf_clear( c->absolute_path );
-            ibuf_add ( c->absolute_path, path );
-            ibuf_clear( c->line_number );
-            ibuf_add ( c->line_number, "1" );
-      } else { 
-		  struct tgdb_source_file *tsf = (struct tgdb_source_file *)
-			  xmalloc ( sizeof ( struct tgdb_source_file ) );
-		  tsf->absolute_path = strdup ( path );
-          tgdb_types_append_command(list, TGDB_ABSOLUTE_SOURCE_ACCEPTED, tsf);
-      }
-   /* not found */
-   } else {
-	  struct tgdb_source_file *rejected = (struct tgdb_source_file *)
-		  xmalloc ( sizeof ( struct tgdb_source_file ) );
+    response = (struct tgdb_response *)
+      xmalloc (sizeof (struct tgdb_response));
+    response->header = TGDB_FILENAME_PAIR;
+    response->choice.filename_pair.absolute_path = strdup (apath);
+    response->choice.filename_pair.relative_path = strdup (rpath);
+    tgdb_types_append_command(list, response);
+  /* not found */
+  } else {
+    struct tgdb_source_file *rejected = (struct tgdb_source_file *)
+      xmalloc ( sizeof ( struct tgdb_source_file ) );
+    struct tgdb_response *response = (struct tgdb_response *)
+      xmalloc (sizeof (struct tgdb_response));
+    response->header = TGDB_ABSOLUTE_SOURCE_DENIED;
 
-	  if ( c->last_info_source_requested == NULL )
-		  rejected->absolute_path = NULL;
-	  else
-		  rejected->absolute_path = strdup ( ibuf_get ( c->last_info_source_requested ) );
-      tgdb_types_append_command(list, TGDB_ABSOLUTE_SOURCE_DENIED, rejected);
-   }
+    if (c->last_info_source_requested == NULL)
+      rejected->absolute_path = NULL;
+    else
+      rejected->absolute_path = strdup ( ibuf_get ( c->last_info_source_requested ) );
+
+    response->choice.absolute_source_denied.source_file = rejected;
+    tgdb_types_append_command(list, response);
+  }
 }
 
-static void commands_process_source_line(struct commands *c ){
-    unsigned long length = ibuf_length(c->info_sources_string), i, start = 0;
-    static char *info_ptr; 
-    static char *nfile;
-    info_ptr = ibuf_get(c->info_sources_string);
+static void
+commands_send_source_relative_source_file (struct commands *c,
+					   struct tgdb_list *list)
+{
+  /* So far, INFO_SOURCE_RELATIVE is only used when a 
+   * TGDB_UPDATE_FILE_POSITION is needed.
+   */
+  /* This section allocates a new structure to add into the queue 
+   * All of its members will need to be freed later.
+   */
+  struct tgdb_file_position *tfp = (struct tgdb_file_position *)
+    xmalloc ( sizeof ( struct tgdb_file_position ) );
+  struct tgdb_response *response = (struct tgdb_response *)
+    xmalloc (sizeof (struct tgdb_response));
+  tfp->absolute_path = strdup ( ibuf_get ( c->absolute_path ) );
+  tfp->relative_path = strdup (ibuf_get (c->info_source_relative_path));
+  tfp->line_number   = atoi ( ibuf_get ( c->line_number ) );
   
-    for(i = 0 ; i < length; ++i){
-        if(i > 0 && info_ptr[i - 1] == ',' && info_ptr[i] == ' '){
-            nfile = calloc(sizeof(char),i - start) ;
-            strncpy(nfile, info_ptr + start , i - start - 1);
-            start += ((i + 1) - start); 
-			tgdb_list_append ( c->inferior_source_files, nfile );
-        } else if (i == length - 1 ){
-            nfile = calloc(sizeof(char),i - start + 2);
-            strncpy(nfile, info_ptr + start , i - start + 1);
-			tgdb_list_append ( c->inferior_source_files, nfile );
-        }
-    }
+  response->header = TGDB_UPDATE_FILE_POSITION;
+  response->choice.update_file_position.file_position = tfp;
+
+  tgdb_types_append_command(list, response);
 }
 
 /* commands_process_info_source:
@@ -542,59 +649,69 @@ static void commands_process_source_line(struct commands *c ){
  * This function is capable of parsing the output of 'info source'.
  * It can get both the absolute and relative path to the source file.
  */
-static void commands_process_info_source (
-		struct commands *c, 
-		struct tgdb_list *list, 
-		char a){
-    unsigned long length;
-    static char *info_ptr;
+static void 
+commands_process_info_source (
+  struct commands *c, 
+  struct tgdb_list *list, 
+  char a)
+{
+  unsigned long length;
+  static char *info_ptr;
 
-    if ( c->info_source_ready  ) /* Already found */
-        return;
+  if (c->info_source_ready) /* Already found */
+    return;
     
-    info_ptr = ibuf_get(c->info_source_string);
-    length   = ibuf_length(c->info_source_string);
+  info_ptr = ibuf_get(c->info_source_string);
+  length   = ibuf_length(c->info_source_string);
 
-    if ( a == '\r' )
-        return;
+  if (a == '\r')
+    return;
 
-    if(a == '\n'){ 
-        /* This is the line containing the absolute path to the source file */
-        if ( commands_get_state(c) == INFO_SOURCE_ABSOLUTE && 
-             length >= c->source_prefix_length && 
-             strncmp(info_ptr, c->source_prefix, c->source_prefix_length) == 0 ) {
+  if(a == '\n')
+  {
+    /* This is the line containing the absolute path to the source file */
+    if (length >= c->source_prefix_length && 
+	strncmp(info_ptr, c->source_prefix, c->source_prefix_length) == 0 ) {
+        ibuf_add (c->info_source_absolute_path, info_ptr + c->source_prefix_length);
+      c->info_source_ready = 1;
+      ibuf_clear(c->info_source_string);
 
-             commands_send_source_absolute_source_file ( c, list );
+      /* commands_finalize_command will use the populated data */
 
-            c->info_source_ready = 1;
-        } else if ( /* This is the line contatining the relative path to the source file */
-            commands_get_state(c) == INFO_SOURCE_RELATIVE && 
-            length >= c->source_relative_prefix_length && 
-            strncmp(info_ptr, c->source_relative_prefix, c->source_relative_prefix_length) == 0 ) {
-
-            /* So far, INFO_SOURCE_RELATIVE is only used when a 
-             * TGDB_UPDATE_FILE_POSITION is needed.
-             */
-            {
-				/* This section allocates a new structure to add into the queue 
-				 * All of its members will need to be freed later.
-				 */
-				struct tgdb_file_position *tfp = (struct tgdb_file_position *)
-					xmalloc ( sizeof ( struct tgdb_file_position ) );
-				tfp->absolute_path = strdup ( ibuf_get ( c->absolute_path ) );
-				tfp->relative_path = strdup ( info_ptr + c->source_relative_prefix_length );
-				tfp->line_number   = atoi ( ibuf_get ( c->line_number ) );
-
-                tgdb_types_append_command(list, TGDB_UPDATE_FILE_POSITION, tfp );
-            }
-
-            c->info_source_ready = 1;
-        } else
-            ibuf_clear(c->info_source_string);
+    /* This is the line contatining the relative path to the source file */
+    } else if (length >= c->source_relative_prefix_length && 
+	       strncmp(info_ptr, c->source_relative_prefix, c->source_relative_prefix_length) == 0 ) {
+      ibuf_add (c->info_source_relative_path, info_ptr + c->source_relative_prefix_length);
+      ibuf_clear(c->info_source_string);
     } else
-        ibuf_addchar(c->info_source_string, a);
+      ibuf_clear(c->info_source_string);
+  } else
+    ibuf_addchar(c->info_source_string, a);
 }
 
+static void 
+commands_process_source_line (struct commands *c)
+{
+  unsigned long length = ibuf_length(c->info_sources_string), i, start = 0;
+  static char *info_ptr; 
+  static char *nfile;
+  info_ptr = ibuf_get(c->info_sources_string);
+  
+  for(i = 0 ; i < length; ++i){
+    if(i > 0 && info_ptr[i - 1] == ',' && info_ptr[i] == ' '){
+      nfile = calloc(sizeof(char),i - start) ;
+      strncpy(nfile, info_ptr + start , i - start - 1);
+      start += ((i + 1) - start); 
+      tgdb_list_append ( c->inferior_source_files, nfile );
+    } else if (i == length - 1 ){
+      nfile = calloc(sizeof(char),i - start + 2);
+      strncpy(nfile, info_ptr + start , i - start + 1);
+      tgdb_list_append ( c->inferior_source_files, nfile );
+    }
+  }
+}
+
+
 /* process's source files */
 static void commands_process_sources(struct commands *c, char a){
     static const char *sourcesReadyString = "Source files for which symbols ";
@@ -622,6 +739,28 @@ static void commands_process_sources(struct commands *c, char a){
     }
 }
 
+static void 
+commands_process_completion(struct commands *c ){
+  const char *ptr = ibuf_get (c->tab_completion_string);
+  tgdb_list_append (c->tab_completions, strdup (ptr));
+}
+
+/* process's source files */
+static void 
+commands_process_complete (struct commands *c, char a)
+{
+  ibuf_addchar(c->tab_completion_string, a);
+   
+  if(a == '\n'){
+    ibuf_delchar(c->tab_completion_string); /* remove '\n' and null terminate */
+
+    if (ibuf_length (c->tab_completion_string) > 0)
+      commands_process_completion(c); 
+
+    ibuf_clear(c->tab_completion_string);
+  }
+}
+
 void commands_free(struct commands *c, void *item) {
     free((char*)item);
 }
@@ -629,19 +768,41 @@ void commands_free(struct commands *c, void *item) {
 void commands_send_gui_sources (
 		struct commands *c, 
 		struct tgdb_list *list){
-	/* If the inferior program was not compiled with debug, then no sources
-	 * will be available. If no sources are available, do not return the
-	 * TGDB_UPDATE_SOURCE_FILES command. */
-	if ( tgdb_list_size ( c->inferior_source_files ) > 0 )
-		tgdb_types_append_command ( list, TGDB_UPDATE_SOURCE_FILES, c->inferior_source_files );
+  /* If the inferior program was not compiled with debug, then no sources
+   * will be available. If no sources are available, do not return the
+   * TGDB_UPDATE_SOURCE_FILES command. */
+  if (tgdb_list_size ( c->inferior_source_files ) > 0) {
+    struct tgdb_response *response = (struct tgdb_response *)
+      xmalloc (sizeof (struct tgdb_response));
+    response->header = TGDB_UPDATE_SOURCE_FILES;
+    response->choice.update_source_files.source_files = c->inferior_source_files;
+    tgdb_types_append_command (list, response);
+  }
+}
+
+
+void commands_send_gui_completions (
+		struct commands *c, 
+		struct tgdb_list *list){
+  /* If the inferior program was not compiled with debug, then no sources
+   * will be available. If no sources are available, do not return the
+   * TGDB_UPDATE_SOURCE_FILES command. */
+//  if (tgdb_list_size ( c->tab_completions ) > 0)
+    struct tgdb_response *response = (struct tgdb_response *)
+      xmalloc (sizeof (struct tgdb_response));
+    response->header = TGDB_UPDATE_COMPLETIONS;
+    response->choice.update_completions.completion_list = c->tab_completions;
+    tgdb_types_append_command (list, response);
 }
 
 void commands_process ( struct commands *c, char a, struct tgdb_list *list){
     if(commands_get_state(c) == INFO_SOURCES){
         commands_process_sources(c, a);     
+    } else if(commands_get_state(c) == COMPLETE){
+        commands_process_complete(c, a);     
     } else if(commands_get_state(c) == INFO_LIST){
         /* do nothing with data */
-    } else if(commands_get_state(c) == INFO_SOURCE_ABSOLUTE 
+    } else if(commands_get_state(c) == INFO_SOURCE_FILENAME_PAIR 
             ||commands_get_state(c) == INFO_SOURCE_RELATIVE){
         commands_process_info_source(c, list, a);   
     } else if(c->breakpoint_table && c->cur_command_state == FIELD && c->cur_field_num == 5){ /* the file name and line num */ 
@@ -674,7 +835,10 @@ static void commands_prepare_info_breakpoints( struct commands *c ) {
  * This prepares the tab completion command
  */
 static void commands_prepare_tab_completion ( struct annotate_two *a2, struct commands *c ) {
-    data_set_state ( a2, USER_COMMAND );
+  c->tab_completion_ready = 0;
+  ibuf_clear(c->tab_completion_string);
+  commands_set_state(c, COMPLETE, NULL);
+  global_set_start_completion ( a2->g );
 }
 
 /* commands_prepare_info_sources: 
@@ -702,26 +866,37 @@ static void commands_prepare_list ( struct annotate_two *a2, struct commands *c,
    c->info_source_ready = 0;
 }
 
-void commands_finalize_command ( 
-		struct commands *c, 
-		struct tgdb_list *list ) {
-    switch ( commands_get_state (c) ) {
-        case INFO_SOURCE_RELATIVE:
-        case INFO_SOURCE_ABSOLUTE:
+void 
+commands_finalize_command ( 
+  struct commands *c, 
+  struct tgdb_list *list )
+{
+  switch ( commands_get_state (c) ) {
+    case INFO_SOURCE_RELATIVE:
+    case INFO_SOURCE_FILENAME_PAIR:
+      if ( c->info_source_ready == 0 ) {
+	struct tgdb_source_file *rejected = (struct tgdb_source_file *)
+	  xmalloc ( sizeof ( struct tgdb_source_file ) );
+	struct tgdb_response *response = (struct tgdb_response *)
+	  xmalloc (sizeof (struct tgdb_response));
+	if (c->last_info_source_requested == NULL)
+	  rejected->absolute_path = NULL;
+	else
+	  rejected->absolute_path = strdup ( ibuf_get ( c->last_info_source_requested ) );
 
-            if ( c->info_source_ready == 0 ) {
-				struct tgdb_source_file *rejected = (struct tgdb_source_file *)
-				  xmalloc ( sizeof ( struct tgdb_source_file ) );
-				if ( c->last_info_source_requested == NULL )
-					rejected->absolute_path = NULL;
-				else
-					rejected->absolute_path = strdup ( ibuf_get ( c->last_info_source_requested ) );
-                tgdb_types_append_command(list, TGDB_ABSOLUTE_SOURCE_DENIED, rejected );
-            }
-
-            break;
-        default: break;
-    }
+	response->header = TGDB_ABSOLUTE_SOURCE_DENIED;
+	response->choice.absolute_source_denied.source_file = rejected;
+	tgdb_types_append_command(list, response);
+      } else {
+	if (commands_get_state (c) == INFO_SOURCE_RELATIVE)
+	  commands_send_source_relative_source_file (c, list);
+	else if (commands_get_state (c) == INFO_SOURCE_FILENAME_PAIR)
+	  commands_send_source_absolute_source_file (c, list);
+      }
+      break;
+    default: 
+      break;
+  }
 }
 
 int commands_prepare_for_command ( 
@@ -754,11 +929,13 @@ int commands_prepare_for_command (
         case ANNOTATE_LIST:
             commands_prepare_list ( a2, c, com->tgdb_client_command_data );
             break;
+        case ANNOTATE_INFO_LINE:
+            break;
         case ANNOTATE_INFO_SOURCE_RELATIVE:
             commands_prepare_info_source ( a2, c, INFO_SOURCE_RELATIVE );
             break;
-        case ANNOTATE_INFO_SOURCE_ABSOLUTE:
-            commands_prepare_info_source ( a2, c, INFO_SOURCE_ABSOLUTE );
+        case ANNOTATE_INFO_SOURCE_FILENAME_PAIR:
+            commands_prepare_info_source ( a2, c, INFO_SOURCE_FILENAME_PAIR);
             break;
         case ANNOTATE_INFO_BREAKPOINTS:
             commands_prepare_info_breakpoints (c);
@@ -768,10 +945,9 @@ int commands_prepare_for_command (
         case ANNOTATE_COMPLETE:
             commands_prepare_tab_completion ( a2, c );
             io_debug_write_fmt("<%s\n>", com->tgdb_client_command_data);
-            return 0;
             break;  /* Nothing to do */
-		case ANNOTATE_INFO_SOURCE:
-		case ANNOTATE_SET_PROMPT:
+	case ANNOTATE_INFO_SOURCE:
+	case ANNOTATE_SET_PROMPT:
         case ANNOTATE_VOID:
 			break;
         default:
@@ -798,92 +974,96 @@ int commands_prepare_for_command (
  * A command ready to be run through the debugger or NULL on error.
  * The memory is malloc'd, and must be freed.
  */
-static char *commands_create_command ( 
-	struct commands *c,
-    enum annotate_commands com,
-    const char *data) {
-
-    char *ncom = NULL;
+static char *
+commands_create_command (struct commands *c, enum annotate_commands com, 
+	                 const char *data)
+{
+  char *ncom = NULL;
     
-    switch ( com ) {
-        case ANNOTATE_INFO_SOURCES: 
-            ncom = strdup ( "server info sources\n" );   
-            break;
-        case ANNOTATE_LIST:
-            {
-				struct ibuf *temp_file_name = NULL;
-                if ( data != NULL ) {
-					temp_file_name = ibuf_init ();
-					ibuf_add ( temp_file_name, data );
-				}
+  switch (com)
+  {
+    case ANNOTATE_INFO_SOURCES: 
+      ncom = strdup ( "server info sources\n" );   
+      break;
+    case ANNOTATE_LIST:
+    {
+      struct ibuf *temp_file_name = NULL;
+      if (data != NULL)
+      {
+	temp_file_name = ibuf_init ();
+	ibuf_add ( temp_file_name, data );
+      }
+      if (data == NULL)
+	ncom = (char *)xmalloc( sizeof ( char ) * ( 16 ));
+      else
+        ncom = (char *)xmalloc( sizeof ( char ) * ( 16 + strlen (data )));
+      strcpy ( ncom, "server list " );
 
-                if ( data == NULL )
-                    ncom = (char *)xmalloc( sizeof ( char ) * ( 16 ));
-                else
-                    ncom = (char *)xmalloc( sizeof ( char ) * ( 16 + strlen (data )));
-                strcpy ( ncom, "server list " );
+      if (temp_file_name != NULL)
+      {
+        strcat (ncom, ibuf_get (temp_file_name));
+        strcat (ncom, ":1");
+      } 
 
-                /* This should only happen for the initial 'list' */
-                if ( temp_file_name != NULL ) {
-                    strcat ( ncom, ibuf_get ( temp_file_name ) );
-                    strcat ( ncom, ":1" );
-                } 
+      /* This happens when the user wants to get the absolute path of 
+       * the current file. They pass in NULL to represent that. */
+      if (temp_file_name == NULL)
+      {
+	ibuf_free ( c->last_info_source_requested );
+	c->last_info_source_requested = NULL;
+      } else {
+	if ( c->last_info_source_requested == NULL )
+	  c->last_info_source_requested = ibuf_init ();
 
-                if ( temp_file_name == NULL ) {
-					ibuf_free ( c->last_info_source_requested );
-					c->last_info_source_requested = NULL;
-				} else {
-					if ( c->last_info_source_requested == NULL )
-						c->last_info_source_requested = ibuf_init ();
+	ibuf_clear ( c->last_info_source_requested );
+	ibuf_add ( c->last_info_source_requested, ibuf_get ( temp_file_name ) );
+      }
 
-					ibuf_clear ( c->last_info_source_requested );
-					ibuf_add ( c->last_info_source_requested, ibuf_get ( temp_file_name ) );
-				}
+      strcat ( ncom, "\n" );
 
-                strcat ( ncom, "\n" );
+      ibuf_free ( temp_file_name );
+      temp_file_name = NULL;
+      break;
+    }
+    case ANNOTATE_INFO_LINE:
+      ncom = strdup ( "server info line\n" );   
+      break;
+    case ANNOTATE_INFO_SOURCE_RELATIVE:                         
+    case ANNOTATE_INFO_SOURCE_FILENAME_PAIR:                         
+      ncom = strdup ( "server info source\n" );   
+      break;
+    case ANNOTATE_INFO_BREAKPOINTS:
+      ncom = strdup ( "server info breakpoints\n" );   
+      break;
+    case ANNOTATE_TTY:
+    {
+      struct ibuf *temp_tty_name = ibuf_init ();
+      ibuf_add ( temp_tty_name, data );
+      ncom = (char *)xmalloc( sizeof ( char ) * ( 13 + strlen( data )));
+      strcpy ( ncom, "server tty " );
+      strcat ( ncom, ibuf_get  (temp_tty_name ) );
+      strcat ( ncom, "\n" );
 
-				ibuf_free ( temp_file_name );
-				temp_file_name = NULL;
-                break;
-            }
-        case ANNOTATE_INFO_SOURCE_RELATIVE:                         
-            ncom = strdup ( "server info source\n" );   
-            break;
-        case ANNOTATE_INFO_SOURCE_ABSOLUTE:                         
-            ncom = strdup ( "server info source\n" );   
-            break;
-        case ANNOTATE_INFO_BREAKPOINTS:
-            ncom = strdup ( "server info breakpoints\n" );   
-            break;
-        case ANNOTATE_TTY:
-            {
-				struct ibuf *temp_tty_name = ibuf_init ();
-				ibuf_add ( temp_tty_name, data );
-                ncom = (char *)xmalloc( sizeof ( char ) * ( 13 + strlen( data )));
-                strcpy ( ncom, "server tty " );
-                strcat ( ncom, ibuf_get  (temp_tty_name ) );
-                strcat ( ncom, "\n" );
+      ibuf_free ( temp_tty_name );
+      temp_tty_name = NULL;
+      break;
+    }
+    case ANNOTATE_COMPLETE:
+      ncom = (char *)xmalloc( sizeof ( char ) * ( 18 + strlen( data )));
+      strcpy ( ncom, "server complete " );
+      strcat ( ncom, data );
+      strcat ( ncom, "\n" );
+      break;
+    case ANNOTATE_SET_PROMPT:
+      ncom = strdup ( data );   
+      break;
+    case ANNOTATE_VOID:
+    default: 
+      logger_write_pos ( logger, __FILE__, __LINE__, "switch error");
+      break;
+  };
 
-				ibuf_free ( temp_tty_name );
-				temp_tty_name = NULL;
-                break;
-            }
-        case ANNOTATE_COMPLETE:
-            ncom = (char *)xmalloc( sizeof ( char ) * ( 18 + strlen( data )));
-            strcpy ( ncom, "server complete " );
-            strcat ( ncom, data );
-            strcat ( ncom, "\n" );
-            break;
-        case ANNOTATE_SET_PROMPT:
-            ncom = strdup ( data );   
-            break;
-        case ANNOTATE_VOID:
-        default: 
-            logger_write_pos ( logger, __FILE__, __LINE__, "switch error");
-            break;
-    };
-
-    return ncom;
+  return ncom;
 }
 
 int commands_user_ran_command ( 
@@ -932,18 +1112,11 @@ int commands_issue_command (
                 TGDB_CLIENT_COMMAND_DISPLAY_NOTHING,
 				TGDB_CLIENT_COMMAND_ACTION_NONE,
                 (void*) nacom ); 
-    } else if ( oob == 2 ) {
-        client_command = tgdb_client_command_create ( 
-                ncom,
-                TGDB_CLIENT_COMMAND_TGDB_BASE,
-                TGDB_CLIENT_COMMAND_DISPLAY_NOTHING,
-                TGDB_CLIENT_COMMAND_ACTION_CONSOLE_SET_PROMPT,
-                NULL ); 
     } else if ( oob == 4 ) {
         client_command = tgdb_client_command_create ( 
                 ncom,
                 TGDB_CLIENT_COMMAND_NORMAL,
-                TGDB_CLIENT_COMMAND_DISPLAY_COMMAND_AND_RESULTS,
+                TGDB_CLIENT_COMMAND_DISPLAY_NOTHING,
                 TGDB_CLIENT_COMMAND_ACTION_NONE,
                 (void*) nacom ); 
     }
