@@ -372,24 +372,32 @@ struct tgdb_list *tgdb_client_get_client_commands (
 //@{
 
 /** 
- * Gets the absolute path of a relative path outputted by a debugger.
+ * Get's the absolute and relative path that relates to the path PATH.
  *  
  * \param tcc
  * The client context.
  *
- * \param relative_path
- * The path that the debugger outputted. ( Usually relative ).
+ * \param path
+ * The path that the debugger outputted. (relative or absolute)
  *
  * @return
  * 0 on success, otherwise -1 on error.
- *
- * \pre
- * If relative_path is absolute ( not relative ), then this function should not
- * be called.
  */
-int tgdb_client_get_absolute_path ( 
+int tgdb_client_get_filename_pair ( 
 		struct tgdb_client_context *tcc, 
-		const char *relative_path );
+		const char *path );
+
+/**
+ * Get's the current fullname, filename and line number that the debugger is 
+ * at.
+ *
+ * \param tcc
+ * The client context.
+ *
+ * @return
+ * 0 on success, otherwise -1 on error.
+ */
+int tgdb_client_get_current_location (struct tgdb_client_context *tcc);
 
 /** 
  * Gets all of the source files that the inferior is made of.
@@ -402,22 +410,6 @@ int tgdb_client_get_absolute_path (
  */
 int tgdb_client_get_inferior_source_files ( struct tgdb_client_context *tcc );
 
-/** 
- * This will change the prompt the user sees to PROMPT.
- *
- * \param tcc
- * The client context.
- *
- * \param prompt
- * The new prompt to change to.
- *
- * @return
- * 0 on success, otherwise -1 on error.
- */
-int tgdb_client_change_debugger_prompt(
-		struct tgdb_client_context *tcc,
-		const char *prompt);
-
 /** 
  * TGDB calls this function when it determines a command needs to be completed.
  *
@@ -524,7 +516,7 @@ int tgdb_client_open_new_tty (
  * @return
  * tty name on success, otherwise NULL on error.
  */
-char *tgdb_client_get_tty_name ( struct tgdb_client_context *tcc );
+const char *tgdb_client_get_tty_name ( struct tgdb_client_context *tcc );
 
 //@}
 
