@@ -63,17 +63,15 @@ static struct tgdb_client_debugger_interfaces {
 	struct tgdb_list *(*tgdb_client_get_client_commands) ( 
 			void *ctx );
 
-	int (*tgdb_client_get_absolute_path) ( 
+	int (*tgdb_client_get_filename_pair) ( 
 			void *ctx, 
-			const char *relative_path );
+			const char *path );
+
+	int (*tgdb_client_get_current_location) (void *ctx);
 
 	int (*tgdb_client_get_inferior_source_files) ( 
 			void *ctx );
 
-	int (*tgdb_client_change_debugger_prompt) (
-			void *ctx,
-			const char *prompt);
-
 	int (*tgdb_client_completion_callback) (
 			void *ctx,
 			const char *completion_command);
@@ -96,7 +94,7 @@ static struct tgdb_client_debugger_interfaces {
 			int *inferior_stdin, 
 			int *inferior_stdout );
 
-	char *(*tgdb_client_get_tty_name) ( 
+	const char *(*tgdb_client_get_tty_name) ( 
 			void *ctx );
 
 } tgdb_client_debugger_interfaces[] = {
@@ -124,12 +122,12 @@ static struct tgdb_client_debugger_interfaces {
 		a2_parse_io,
 		/* tgdb_client_get_client_commands */
 		a2_get_client_commands,
-		/* tgdb_client_get_absolute_path */
-		a2_get_source_absolute_filename,
+		/* tgdb_client_get_filename_pair */
+		a2_get_source_filename_pair,
+		/* tgdb_client_get_current_location */
+		a2_get_current_location,
 		/* tgdb_client_get_inferior_source_files */
 		a2_get_inferior_sources,
-		/* tgdb_client_change_debugger_prompt */
-		a2_change_prompt,
 		/* tgdb_client_completion_callback */
 		a2_completion_callback,
 		/* tgdb_client_return_command*/
@@ -167,12 +165,12 @@ static struct tgdb_client_debugger_interfaces {
 		gdbmi_parse_io,
 		/* tgdb_client_get_client_commands */
 		gdbmi_get_client_commands,
-		/* tgdb_client_get_absolute_path */
+		/* tgdb_client_get_filename_pair */
+		NULL,
+		/* tgdb_client_get_current_location */
 		NULL,
 		/* tgdb_client_get_inferior_source_files */
 		NULL,
-		/* tgdb_client_change_debugger_prompt */
-		NULL,
 		/* tgdb_client_completion_callback */
 		NULL,
 		/* tgdb_client_return_command*/
@@ -208,12 +206,12 @@ static struct tgdb_client_debugger_interfaces {
 		NULL,
 		/* tgdb_client_parse_io */
 		NULL,
-		/* tgdb_client_get_absolute_path */
+		/* tgdb_client_get_filename_pair */
+		NULL,
+		/* tgdb_client_get_current_location */
 		NULL,
 		/* tgdb_client_get_inferior_source_files */
 		NULL,
-		/* tgdb_client_change_debugger_prompt */
-		NULL,
 		/* tgdb_client_completion_callback */
 		NULL,
 		/* tgdb_client_return_command*/
@@ -406,16 +404,27 @@ struct tgdb_list *tgdb_client_get_client_commands (
 			tcc->tgdb_debugger_context );
 }
 
-int tgdb_client_get_absolute_path ( 
+int tgdb_client_get_filename_pair ( 
 		struct tgdb_client_context *tcc, 
-		const char *relative_path ) {
+		const char *path ) {
 	if ( tcc == NULL || tcc->tgdb_client_interface == NULL ) {
 		logger_write_pos ( logger, __FILE__, __LINE__, "tgdb_client_get_absolute_path unimplemented");
 		return -1;
 	}
 	
-	return tcc->tgdb_client_interface->tgdb_client_get_absolute_path ( 
-			tcc->tgdb_debugger_context, relative_path );
+	return tcc->tgdb_client_interface->tgdb_client_get_filename_pair ( 
+			tcc->tgdb_debugger_context, path );
+}
+
+int tgdb_client_get_current_location (struct tgdb_client_context *tcc)
+{
+  if (tcc == NULL || tcc->tgdb_client_interface == NULL)
+  {
+    logger_write_pos ( logger, __FILE__, __LINE__, "tgdb_client_get_current_location unimplemented");
+    return -1;
+  }
+	
+  return tcc->tgdb_client_interface->tgdb_client_get_current_location (tcc->tgdb_debugger_context);
 }
 
 int tgdb_client_get_inferior_source_files ( struct tgdb_client_context *tcc ) {
@@ -428,12 +437,6 @@ int tgdb_client_get_inferior_source_files ( struct tgdb_client_context *tcc ) {
 			tcc->tgdb_debugger_context );
 }
 
-int tgdb_client_change_debugger_prompt(
-		struct tgdb_client_context *tcc,
-		const char *prompt) {
-	return -1;
-}
-
 int tgdb_client_completion_callback(
 		struct tgdb_client_context *tcc,
 		const char *completion_command) {
@@ -498,7 +501,7 @@ int tgdb_client_open_new_tty (
 			inferior_stdin, inferior_stdout );
 }
 
-char *tgdb_client_get_tty_name ( struct tgdb_client_context *tcc ) {
+const char *tgdb_client_get_tty_name ( struct tgdb_client_context *tcc ) {
 	if ( tcc == NULL || tcc->tgdb_client_interface == NULL ) {
 		logger_write_pos ( logger, __FILE__, __LINE__, "tgdb_client_get_tty_name unimplemented" );
 		return NULL;
